#include "Python.h"

#include "DippyDecompAlgo.h"
#include "DippyDecompApp.h"
#include "DippyPythonUtils.h"

/**
 * Perform branching
 *
 * This function should populate the (down|up)Branch(LB|UB) vectors with (indices, bound-value) pairs
 * which define the branch.
 */
bool DippyAlgoMixin::chooseBranchSet(DecompAlgo *algo, std::vector< std::pair<int, double> > &downBranchLB,
        std::vector< std::pair<int, double> > &downBranchUB,
        std::vector< std::pair<int, double> > &upBranchLB,
        std::vector< std::pair<int, double> > &upBranchUB){

	if (!m_utilParam->GetSetting("pyBranchMethod", true))
        return algo->DecompAlgo::chooseBranchSet(downBranchLB, downBranchUB, upBranchLB, upBranchUB);

	DippyDecompApp * app = (DippyDecompApp *)algo->getDecompApp();

    // copy the current solution into a Python list
    const double *xhat = algo->getXhat();
    PyObject *pSolutionList = pyTupleList_FromDoubleArray(xhat, app->m_colList);

    // try to call chooseBranchSet on the DipProblem python object
    PyObject* pResult = PyObject_CallMethod(m_pProb, "chooseBranchSet", "O", pSolutionList);

    if (pResult == NULL) { 
        //something's gone wrong with the function call, a Python exception has been set
        throw UtilException("Error calling method prob.chooseBranchSet()", "chooseBranchSet", "DippyDecompAlgo");
    }

    // need more error checking/assertion setting here
    if (pResult == Py_None) {
        // if chooseBranchSet returns None, do default branching for this algo
        return algo->DecompAlgo::chooseBranchSet(downBranchLB, downBranchUB, upBranchLB, upBranchUB);
    }

    // or else, the function should have returned 4 lists of (name, value) tuples
    PyObject* pDownLB = PyTuple_GetItem(pResult, 0);
    PyObject* pDownUB = PyTuple_GetItem(pResult, 1);
    PyObject* pUpLB = PyTuple_GetItem(pResult, 2);
    PyObject* pUpUB = PyTuple_GetItem(pResult, 3);

    // copy the python dictionaries into the result vectors
	pyColDict_AsPairedVector(pDownLB, downBranchLB, app->m_colIndices);
    pyColDict_AsPairedVector(pDownUB, downBranchUB, app->m_colIndices);
    pyColDict_AsPairedVector(pUpLB, upBranchLB, app->m_colIndices);
    pyColDict_AsPairedVector(pUpUB, upBranchUB, app->m_colIndices);

    return true;
}
